@@ -28,8 +28,8 @@ import java.security.PrivilegedExceptionAction;
 import java.util.HashMap;
 import java.util.Map;
 
-import javax.servlet.WriteListener;
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.WriteListener;
+import jakarta.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.Globals;
 import org.apache.coyote.ActionCode;
@@ -40,8 +40,9 @@ import org.apache.tomcat.util.buf.C2BConverter;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * The buffer used by Tomcat response. This is a derivative of the Tomcat 3.3 OutputBuffer, with the removal of some of
- * the state handling (which in Coyote is mostly the Processor's responsibility).
+ * The buffer used by Tomcat response. This is a derivative of the Tomcat 3.3
+ * OutputBuffer, with the removal of some of the state handling (which in
+ * Coyote is mostly the Processor's responsibility).
  *
  * @author Costin Manolache
  * @author Remy Maucherat
@@ -55,7 +56,7 @@ public class OutputBuffer extends Writer {
     /**
      * Encoder cache.
      */
-    private final Map<Charset,C2BConverter> encoders = new HashMap<>();
+    private final Map<Charset, C2BConverter> encoders = new HashMap<>();
 
 
     /**
@@ -212,7 +213,8 @@ public class OutputBuffer extends Writer {
 
 
     /**
-     * Close the output buffer. This tries to calculate the response size if the response has not been committed yet.
+     * Close the output buffer. This tries to calculate the response size if
+     * the response has not been committed yet.
      *
      * @throws IOException An underlying IOException occurred
      */
@@ -232,18 +234,23 @@ public class OutputBuffer extends Writer {
             flushCharBuffer();
         }
 
-        // Content length can be calculated if:
-        // - the response has not been committed
-        // AND
-        // - the content length has not been explicitly set
-        // AND
-        // - some content has been written OR this is NOT a HEAD request
-        if ((!coyoteResponse.isCommitted()) && (coyoteResponse.getContentLengthLong() == -1) &&
-                ((bb.remaining() > 0 || !coyoteResponse.getRequest().method().equals("HEAD")))) {
-            coyoteResponse.setContentLength(bb.remaining());
+        if ((!coyoteResponse.isCommitted()) && (coyoteResponse.getContentLengthLong() == -1)
+                && !coyoteResponse.getRequest().method().equals("HEAD")) {
+            // If this didn't cause a commit of the response, the final content
+            // length can be calculated. Only do this if this is not a HEAD
+            // request since in that case no body should have been written and
+            // setting a value of zero here will result in an explicit content
+            // length of zero being set on the response.
+            if (!coyoteResponse.isCommitted()) {
+                coyoteResponse.setContentLength(bb.remaining());
+            }
         }
 
-        doFlush(coyoteResponse.getStatus() == HttpServletResponse.SC_SWITCHING_PROTOCOLS);
+        if (coyoteResponse.getStatus() == HttpServletResponse.SC_SWITCHING_PROTOCOLS) {
+            doFlush(true);
+        } else {
+            doFlush(false);
+        }
         closed = true;
 
         // The request should have been completely read by the time the response
@@ -271,7 +278,6 @@ public class OutputBuffer extends Writer {
      * Flush bytes or chars contained in the buffer.
      *
      * @param realFlush <code>true</code> if this should also cause a real network flush
-     *
      * @throws IOException An underlying IOException occurred
      */
     protected void doFlush(boolean realFlush) throws IOException {
@@ -283,7 +289,7 @@ public class OutputBuffer extends Writer {
         try {
             doFlush = true;
             if (initial) {
-                coyoteResponse.commit();
+                coyoteResponse.sendHeaders();
                 initial = false;
             }
             if (cb.remaining() > 0) {
@@ -311,7 +317,8 @@ public class OutputBuffer extends Writer {
     // ------------------------------------------------- Bytes Handling Methods
 
     /**
-     * Sends the buffer data to the client output, checking the state of Response and calling the right interceptors.
+     * Sends the buffer data to the client output, checking the
+     * state of Response and calling the right interceptors.
      *
      * @param buf the ByteBuffer to be written to the response
      *
@@ -322,6 +329,9 @@ public class OutputBuffer extends Writer {
         if (closed) {
             return;
         }
+        if (coyoteResponse == null) {
+            return;
+        }
 
         // If we really have something to write
         if (buf.remaining() > 0) {
@@ -329,24 +339,25 @@ public class OutputBuffer extends Writer {
             try {
                 coyoteResponse.doWrite(buf);
             } catch (CloseNowException e) {
-                // Catch this subclass as it requires specific handling.
+                // Catch this sub-class as it requires specific handling.
                 // Examples where this exception is thrown:
                 // - HTTP/2 stream timeout
                 // Prevent further output for this response
                 closed = true;
                 throw e;
-            } catch (IOException ioe) {
+            } catch (IOException e) {
                 // An IOException on a write is almost always due to
                 // the remote client aborting the request. Wrap this
                 // so that it can be handled better by the error dispatcher.
-                throw new ClientAbortException(ioe);
+                coyoteResponse.setErrorException(e);
+                throw new ClientAbortException(e);
             }
         }
 
     }
 
 
-    public void write(byte[] b, int off, int len) throws IOException {
+    public void write(byte b[], int off, int len) throws IOException {
 
         if (suspended) {
             return;
@@ -368,7 +379,7 @@ public class OutputBuffer extends Writer {
     }
 
 
-    private void writeBytes(byte[] b, int off, int len) throws IOException {
+    private void writeBytes(byte b[], int off, int len) throws IOException {
 
         if (closed) {
             return;
@@ -392,9 +403,8 @@ public class OutputBuffer extends Writer {
             return;
         }
 
-        int remaining = from.remaining();
         append(from);
-        bytesWritten += remaining;
+        bytesWritten += from.remaining();
 
         // if called from within flush(), then immediately flush
         // remaining bytes
@@ -441,12 +451,12 @@ public class OutputBuffer extends Writer {
             }
             if (from.remaining() > 0) {
                 flushByteBuffer();
-            } else if (conv.isUnderflow() && bb.limit() > bb.capacity() - 4) {
+            } else if (conv.isUndeflow() && bb.limit() > bb.capacity() - 4) {
                 // Handle an edge case. There are no more chars to write at the
                 // moment but there is a leftover character in the converter
                 // which must be part of a surrogate pair. The byte buffer does
                 // not have enough space left to output the bytes for this pair
-                // once it is complete (it will require 4 bytes) so flush now to
+                // once it is complete )it will require 4 bytes) so flush now to
                 // prevent the bytes for the leftover char and the rest of the
                 // surrogate pair yet to be written from being lost.
                 // See TestOutputBuffer#testUtf8SurrogateBody()
@@ -474,7 +484,7 @@ public class OutputBuffer extends Writer {
 
 
     @Override
-    public void write(char[] c) throws IOException {
+    public void write(char c[]) throws IOException {
 
         if (suspended) {
             return;
@@ -486,7 +496,7 @@ public class OutputBuffer extends Writer {
 
 
     @Override
-    public void write(char[] c, int off, int len) throws IOException {
+    public void write(char c[], int off, int len) throws IOException {
 
         if (suspended) {
             return;
@@ -545,7 +555,11 @@ public class OutputBuffer extends Writer {
             return;
         }
 
-        Charset charset = coyoteResponse.getCharset();
+        Charset charset = null;
+
+        if (coyoteResponse != null) {
+            charset = coyoteResponse.getCharset();
+        }
 
         if (charset == null) {
             if (coyoteResponse.getCharacterEncoding() != null) {
@@ -583,7 +597,7 @@ public class OutputBuffer extends Writer {
     }
 
 
-    // -------------------- BufferedOutputStream compatibility
+    // --------------------  BufferedOutputStream compatibility
 
     public long getContentWritten() {
         return bytesWritten + charsWritten;
@@ -592,7 +606,8 @@ public class OutputBuffer extends Writer {
     /**
      * Has this buffer been used at all?
      *
-     * @return true if no chars or bytes have been added to the buffer since the last call to {@link #recycle()}
+     * @return true if no chars or bytes have been added to the buffer since the
+     *         last call to {@link #recycle()}
      */
     public boolean isNew() {
         return (bytesWritten == 0) && (charsWritten == 0);
@@ -632,8 +647,8 @@ public class OutputBuffer extends Writer {
 
 
     /*
-     * All the non-blocking write state information is held in the Response so it is visible / accessible to all the
-     * code that needs it.
+     * All the non-blocking write state information is held in the Response so
+     * it is visible / accessible to all the code that needs it.
      */
 
     public boolean isReady() {
@@ -660,10 +675,9 @@ public class OutputBuffer extends Writer {
      * @param src Bytes array
      * @param off Offset
      * @param len Length
-     *
      * @throws IOException Writing overflow data to the output channel failed
      */
-    public void append(byte[] src, int off, int len) throws IOException {
+    public void append(byte src[], int off, int len) throws IOException {
         if (bb.remaining() == 0) {
             appendByteArray(src, off, len);
         } else {
@@ -679,16 +693,14 @@ public class OutputBuffer extends Writer {
 
     /**
      * Add data to the buffer.
-     *
      * @param src Char array
      * @param off Offset
      * @param len Length
-     *
      * @throws IOException Writing overflow data to the output channel failed
      */
-    public void append(char[] src, int off, int len) throws IOException {
+    public void append(char src[], int off, int len) throws IOException {
         // if we have limit and we're below
-        if (len <= cb.capacity() - cb.limit()) {
+        if(len <= cb.capacity() - cb.limit()) {
             transfer(src, off, len, cb);
             return;
         }
@@ -699,11 +711,11 @@ public class OutputBuffer extends Writer {
         // copy the first part, flush, then copy the second part - 1 write
         // and still have some space for more. We'll still have 2 writes, but
         // we write more on the first.
-        if (len + cb.limit() < 2 * cb.capacity()) {
-            /*
-             * If the request length exceeds the size of the output buffer, flush the output buffer and then write the
-             * data directly. We can't avoid 2 writes, but we can write more on the second
-             */
+        if(len + cb.limit() < 2 * cb.capacity()) {
+            /* If the request length exceeds the size of the output buffer,
+               flush the output buffer and then write the data directly.
+               We can't avoid 2 writes, but we can write more on the second
+            */
             int n = transfer(src, off, len, cb);
 
             flushCharBuffer();
@@ -731,13 +743,7 @@ public class OutputBuffer extends Writer {
         }
     }
 
-
-    public void setErrorException(Exception e) {
-        coyoteResponse.setErrorException(e);
-    }
-
-
-    private void appendByteArray(byte[] src, int off, int len) throws IOException {
+    private void appendByteArray(byte src[], int off, int len) throws IOException {
         if (len == 0) {
             return;
         }
@@ -846,19 +852,23 @@ public class OutputBuffer extends Writer {
     }
 
     private void toReadMode(Buffer buffer) {
-        buffer.limit(buffer.position()).reset();
+        buffer.limit(buffer.position())
+              .reset();
     }
 
     private void toWriteMode(Buffer buffer) {
-        buffer.mark().position(buffer.limit()).limit(buffer.capacity());
+        buffer.mark()
+              .position(buffer.limit())
+              .limit(buffer.capacity());
     }
 
 
-    private static class PrivilegedCreateConverter implements PrivilegedExceptionAction<C2BConverter> {
+    private static class PrivilegedCreateConverter
+            implements PrivilegedExceptionAction<C2BConverter> {
 
         private final Charset charset;
 
-        PrivilegedCreateConverter(Charset charset) {
+        public PrivilegedCreateConverter(Charset charset) {
             this.charset = charset;
         }
 
